from cost_model import *
import math
import sys

def df_cost_conc_fully_balanced(p):
	p = int(p)
	k = 4*p-1
	
	a = 2*p
	h = p
	g = a*h+1

	N_r = a*(a*h+1)
	N = a*p*(a*h+1)
	k_r = k - p

	#get the cost of routers
	E_r = N_r*rt(k)

	#get the cost of electric cables inside groups
	N_l_c = g*(a*(a-1)/2.0)
	E_i = N_l_c * el(1)

	#get the cost of global cables
	dims = get_decomposition(g)

	[E_o,N_g_c] = get_global_cables_cost_df(dims,1)

	final_cost = E_r+E_i+E_o

	final_power = N*nic_power() + N_r*router_power(k)

	N_el = N_l_c
	N_op = N_g_c
	E_l = E_i
	E_g = E_o

	if((N >= 8000 and N <= 12000) or (k >= 41 and k <= 46)):
		print(">>>>>>>>>>>> DF (fixed p, fully balanced)")
		print("\tN: %d, N_r: %d, k: %d, k': %d, p: %d" % (N, N_r, k, k_r, p))
		print("\tN_el: %d, N_op: %d, N_l: %d, N_g: %d" % (N_el, N_op, N_l_c, N_g_c))
		print("\ta: %d, g: %d, h: %d" % (a, g, h))
		print("\tC-per_endpoint: %d, P-per_endpoint: %f" % (int(final_cost/(1.0*N)), float(final_power/(1.0*N))))
		print("\tC_r: %d, C_el: %d, C_o: %d, C_c: %d" % (int(E_r), int(E_l), int(E_g), int(final_cost)))

	return [final_cost,final_power,N_r,N]

#used previously, gives 25% of diff
def df_cost_relaxed(k,N):
	k = int(k)
	N = int(N)
	N_origin = N
	
	if((k+1) % 4 != 0):
		if((k+2) % 4 == 0):
			k = k+1
			p = (k+2)/4.0
		elif(k % 4 == 0):
			k = k-1
			p = k/4.0
		else:
			print 'radix error'
			return [-1,-1,-1,N]
	else:
		#now we have a perfect DF
		p = (k+1)/4.0

	p = int(p)

	g = int(math.ceil(N/(2.0*p*p)))

	difference = 10000000
	a_final = 0
	h_final = 0

	for a in range(1,6*p):
		for h in range(1,6*p):
			N = a*p*(a*h+1)
			if((abs(N - N_origin) < difference) and (p >= h) and (a >= 2*h)):
				difference = abs(N - N_origin)
				a_final = a
				h_final = h

	#a = 2*p
	#h = p
	a = a_final
	h = h_final
	N_r = a*(a*h+1)
	N = a*p*(a*h+1)
	k_r = k - p

	#get the cost of routers

	E_r = N_r*rt(k)

	#get the cost of electric cables inside groups

	N_l_c = g*(a*(a-1)/2.0)
	E_i = g * (a*(a-1)/2.0) * el(1)

	#get the cost of global cables

	dims = get_decomposition(g)

	[E_o,N_g_c] = get_global_cables_cost_df(dims,1)

	final_cost = E_r+E_i+E_o

	final_power = N*nic_power() + N_r*router_power(k)

	N_el = N_l_c
	N_op = N_g_c
	E_l = E_i
	E_g = E_o

	print(">>>>>>>>>>>> DF (k and N)")
	print("\tN: %d, N_r: %d, k: %d, k': %d, p: %d" % (N, N_r, k, k_r, p))
	print("\tN_el: %d, N_op: %d, N_l: %d, N_g: %d" % (N_el, N_op, N_l_c, N_g_c))
	print("\ta: %d, g: %d, h: %d" % (a, g, h))
	print("\tC-per_endpoint: %d, P-per_endpoint: %f" % (int(final_cost/(1.0*N)), float(final_power/(1.0*N))))
	print("\tC_r: %d, C_el: %d, C_o: %d, C_c: %d" % (int(E_r), int(E_l), int(E_g), int(final_cost)))

	#print(">>>> DF: %d routers, %d groups; group size: %d, endpoints (computed): %d, k (computed): %d, p: %d, cables: %d, global: %d, power: %d" % (N_r, g, a, p*N_r, 4*p-1, p, N_l_c+N_g_c,N_g_c,int(final_power)))
	#print("\t E_r: %d, E_i: %d, E_o: %d,\t\tE_c: %d, E-per_endpoint: %d, P-per_endpoint: %f" % (int(E_r), int(N_l_c), int(N_g_c), int(final_cost),int(final_cost/(1.0*N)), float(final_power/(1.0*N))))

	return [final_cost,final_power,N_r,N]



def df_cost(k,N):
	k = int(k)
	N = int(N)
	
	if((k+1) % 4 != 0):
		if((k+2) % 4 == 0):
			k = k+1
			p = (k+2)/4.0
		elif(k % 4 == 0):
			k = k-1
			p = k/4.0
		else:
			print 'radix error'
			return [-1,-1,-1,N]
	else:
		p = (k+1)/4.0

	g = int(math.ceil(N/(2.0*p*p)))
	a = 2*p
	h = p
	N_r = int(math.ceil(N/(1.0*p)))
	N_r_last_gr = N_r % a

	#get the cost of routers

	E_r = N_r*rt(k)

	#get the cost of electric cables inside groups

	N_l_c = (g-1)*(a*(a-1)/2.0) + 1*(N_r_last_gr*(N_r_last_gr-1)/2.0)

	E_i = (g-1) * (a*(a-1)/2.0) * el(1)
	E_i += 1 * (N_r_last_gr*(N_r_last_gr-1)/2.0) * el(1)

	#get the cost of global cables

	dims = get_decomposition(g)
	#print 'Decomposing ' + str(g) + ' groups. Dimensions: X: ' + str(dims[0]) + ', Y: ' + str(dims[1]) + ', Z: ' + str(dims[2]) + '\n'

	[E_o,N_g_c] = get_global_cables_cost_df(dims,1)
	E_o -= ((a - N_r_last_gr)*h*op(dims[0]/2.0 + 2))/2.0
	N_g_c -= (a - N_r_last_gr)*h/2.0

	final_cost = E_r+E_i+E_o

	final_power = N*nic_power() + N_r

	print(">>>> DF: %d routers, %d groups; group size: %d, endpoints (computed): %d, k (computed): %d, p: %d, cables: %d, global: %d, power: %d" % (N_r, g, a, p*N_r, 4*p-1, p, N_l_c+N_g_c,N_g_c,int(final_power)))

	print("\t E_r: %d, E_i: %d, E_o: %d,\t\tE_c: %d, E-per_endpoint: %d, P-per_endpoint: %f" % (int(E_r), int(N_l_c), int(N_g_c), int(final_cost),int(final_cost/(1.0*N)), float(final_power/(1.0*N))))
	#final_power = N_r*k*4*0.7

	return [final_cost,final_power,N_r,N]

if(len(sys.argv) > 1):
	k = int(sys.argv[1])
	N = int(sys.argv[2])
	cost = df_cost(k,N)
	print 'DF cost (k = ' + str(k) + ', N = ' + str(N) + ') is: ' + str(int(cost)) + ' $\n'

	
	
	
